#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import json
from datetime import datetime
from pathlib import Path
from unittest.mock import patch

from delivery.delivery import main
from delivery.products import ProductMetadata

# The purpose of this suite is to ensure that we implement the delivery directory
# requirements of https://open-confluence.nrao.edu/display/SPR/Delivery+Directory+Improvements


def test_product_metadata():
    metadata = ProductMetadata("EVLA", "13B-014", "execblock", "", "", "23423.23232332")
    assert metadata.choose_template() == ProductMetadata.DIRECTORY_TEMPLATES["NRAO"]
    assert "$" not in metadata.template_to_mustache(ProductMetadata.DIRECTORY_TEMPLATES["NRAO"])
    assert "{" not in metadata.to_path()


def test_delivery_directory_improvements(resource_path_root):
    # the plan here is basically to iterate the list of items in the product_metadata directory
    # and generate each path we want for these and verify them.
    expected = {
        "99509886-dad3-4036-8fff-dfa0535303fb.json": "AGBT14A_110/AGBT14A_110_1234/raw/",
        "cb1ed0a0-3f9d-4abf-84a2-e39dd983adbb.json": "22A-068/imaging_pipeline.0.0/",
        "51f73863-7e54-42a7-a5d1-9c86073b4910.json": "2018.1.01140.S/uid___A001_X133d_X3b22/imaging_pipeline.0.0/",
        "8cf62cf1-3d5d-454f-b356-16d902566251.json": "NAPI/observation.43904.5231713/",
        "85523bcb-dd68-4472-98a4-21fdfcffabf1.json": "12A-339/observation.56065.22587673612/",
        "3a085f1a-aab4-4d41-8682-4ba329fe557b.json": "2021.1.00869.L/uid___A001_X1590_X18a4/calibration_pipeline.unknown_date/",
        "f02a2d91-3fd4-4c7e-ad9c-bbaa014ff4ef.json": "C121/C121A/",
        "233f0f04-675c-466a-8012-5637bb6a649c.json": "21A-423/calibration_pipeline.unknown_date/",
        "24093c45-d766-4ab3-9f88-96b1bc98ee81.json": "VLASS2.2/imaging_pipeline.0.0/",
        "ecfa9dd9-cd51-42f0-b9cb-a305933be0c1.json": "BK114//",
        "f8e28b25-b958-4e5e-a906-b6dd17bcbd23.json": "2019.1.01558.S/uid___A001_X1465_X831/observation.58825.75216611111/",
        "d9a2aed2-99e4-45b7-8e37-11ac7abc9f37.json": "2018.1.01140.S/uid___A001_X133d_X3b22/imaging_pipeline.0.0/",
        "9e9fb126-5af9-42ef-bf3b-d1e7b42d7a30.json": "BB397/BB397CL/",
    }
    for file in (resource_path_root / "product_metadata").glob("*.json"):
        assert expected[file.name] == ProductMetadata.from_json(json.loads(file.read_bytes())).to_path()


def test_tar_file_name_multi(resource_path_root, tmp_path):
    pf_output = resource_path_root / "productfetcher-output-multi"

    with patch("delivery.destinations.tar.datetime", wraps=datetime) as dt:
        dt.now.return_value = datetime(2022, 1, 3, 8, 14, 56)
        main([str(pf_output), "-l", str(tmp_path), "-t"])

    # now we can look at tmp_path and see what sort of filename we got
    archive_filename = f"NRAO_archive_multi_project_20220103-081456.tar"
    assert (tmp_path / archive_filename).exists()


def test_tar_file_name_single(resource_path_root, tmp_path):
    pf_output = resource_path_root / "productfetcher-output-single"
    with patch("delivery.destinations.tar.datetime", wraps=datetime) as dt:
        dt.now.return_value = datetime(2022, 1, 3, 8, 14, 56)
        main([str(pf_output), "-l", str(tmp_path), "-t"])

    archive_filename = f"NRAO_archive_VLASS2.2_20220103-081456.tar"
    assert (tmp_path / archive_filename).exists()


def test_right_number_of_containing_folders(resource_path_root, tmp_path):
    pf_output = resource_path_root / "productfetcher-output-single"
    with patch("delivery.destinations.tar.datetime", wraps=datetime) as dt:
        dt.now.return_value = datetime(2022, 1, 3, 8, 14, 56)
        main([str(pf_output), "-l", str(tmp_path)])

    assert (tmp_path / "VLASS2.2").exists()
    assert (tmp_path / "VLASS2.2").is_dir()

    assert (tmp_path / "VLASS2.2" / "imaging_pipeline.unknown_date").exists()
    assert (tmp_path / "VLASS2.2" / "imaging_pipeline.unknown_date").is_dir()

    assert (
        tmp_path
        / "VLASS2.2"
        / "imaging_pipeline.unknown_date"
        / "VLASS2.2.ql.T01t35.J172711-363000.10.2048.v1.I.iter1.image.pbcor.tt0.subim.fits"
    ).exists()
    assert (
        tmp_path
        / "VLASS2.2"
        / "imaging_pipeline.unknown_date"
        / "VLASS2.2.ql.T01t35.J172711-363000.10.2048.v1.I.iter1.image.pbcor.tt0.subim.fits"
    ).is_file()


def test_right_number_of_containing_folders_for_vla(resource_path_root, tmp_path):
    pf_output = resource_path_root / "productfetcher-output-single-vla"
    with patch("delivery.destinations.tar.datetime", wraps=datetime) as dt:
        dt.now.return_value = datetime(2022, 1, 3, 8, 14, 56)
        main([str(pf_output), "-l", str(tmp_path)])

    testing_path = tmp_path / "VLASS2.2"
    assert (testing_path).exists()
    assert (testing_path).is_dir()

    testing_path = testing_path / "imaging_pipeline.unknown_date"
    assert (testing_path).exists()
    assert (testing_path).is_dir()

    testing_path = testing_path / "VLASS2.2.ql.T01t35.J172711-363000.10.2048.v1.I.iter1.image.pbcor.tt0.subim.fits"
    assert (testing_path).exists()
    assert (testing_path).is_file()
